package safehome.device.netcamera;

import java.awt.image.BufferedImage;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;

import safehome.device.interfaceCamera;

/**
 * This class represents a network connected camera. 
 * <p>
 * A network camera reads camera images from a camera peer and sends
 * control signals to the camera peer through socket communication. 
 * The operations of this class is a subset of a camera device.
 * @author yhkim
 *
 */
public class NetworkCamera implements interfaceCamera, Runnable{


	private Socket socket;
	private DataInputStream inStream;
	private DataOutputStream outStream;
	private BufferedImage img;


	/**
	 * Constructs a network camera instance It connects to SafeHome core
	 * @param safehomeCoreIP the host name of SafeHome core
	 * @param safehomeCorePort the port number
	 * @throws UnknownHostException if the IP address of the host could not be determined. 
	 * @throws IOException if an I/O error occurs when creating the socket.
	 */
	public NetworkCamera(String safehomeCoreIP, int safehomeCorePort) throws UnknownHostException, IOException{
		socket = new Socket(safehomeCoreIP, safehomeCorePort);
		inStream = new DataInputStream(socket.getInputStream());
		outStream = new DataOutputStream(socket.getOutputStream());
		img = new BufferedImage(500, 500, BufferedImage.TYPE_INT_RGB);

	}
	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#getID()
	 * Not yet implemented
	 */
	@Override
	public int getID() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#getView()
	 */
	@Override
	public BufferedImage getView() {
		synchronized(img){
			return img;
		}
	}

	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#panLeft()
	 */
	@Override
	public boolean panLeft() {
		return sendControl(CameraPacket.CONTROL_PAN_LEFT);
		
	}

	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#panRight()
	 */
	@Override
	public boolean panRight() {
		return sendControl(CameraPacket.CONTROL_PAN_RIGHT);
	}

	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#setID(int)
	 * Not yet implemented
	 */
	@Override
	public void setID(int id) {


	}

	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#zoomIn()
	 */
	@Override
	public boolean zoomIn() {
		return sendControl(CameraPacket.CONTROL_ZOOM_IN);
		
	}

	/* (non-Javadoc)
	 * @see safehome.device.interfaceCamera#zoomOut()
	 */
	@Override
	public boolean zoomOut() {
		return sendControl(CameraPacket.CONTROL_ZOOM_OUT);
		
	}
	@Override
	public void run() {
		while(true){
			int size;
			byte[] data = null;
			CameraPacket packet = null;

			try {
				/* First read the size of the incoming packet */
				size = inStream.readInt();
				System.out.println("viewer packet size = " + size);
				//if (size < 0 || size > 80000){ continue; }
				data = new byte[size];
				/* Read size bytes data from socket */
				inStream.readFully(data, 0, size);
				/* Convert the serialized packet to CameraPacket instance */
				packet = CameraPacket.toPacket(data);
			} catch (IOException e) {

				e.printStackTrace();
			}

			System.out.println("packet type = " + packet.getType());
			switch(packet.getType()){
			case CameraPacket.CONTROL_SEND_IMAGE:
				synchronized(img){
					/* Change the camera image */
					img = packet.getBufferedImage();
				}
				break;
			}
		}
	}
	private boolean sendControl(int type){
		CameraPacket packet = new CameraPacket(type);

		System.out.println("viewer: size of control packet, " + packet + " = " + packet.toBytes().length);

		try {
			
			outStream.writeInt(packet.toBytes().length);
			outStream.write(packet.toBytes());
		} catch(SocketException se){
			se.printStackTrace();
			return false;
		}catch (IOException e1) {
			e1.printStackTrace();
			return false;
		} 
		return true;
	}

}
