package safehome.device.netcamera;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import javax.imageio.ImageIO;

/**
 * This class represents a network camera packet.
 * <p>
 * Network camera packets are used for communication between network cameras
 * and a camera viewer. A camera packet consists of a control signal and a camera image.
 * A control signal is pre-defined integer constant. A camera image is a byte array of 
 * <code>BufferedImage</code> in <code>DeviceCamera</code>. A camera packet instance is 
 * converted to byte array before socket communication.
 * @author yhkim
 *
 */
public class CameraPacket implements Serializable {

	//private int size = 0;
	private int type = 0;
	private byte[] serializedImage = null;
	private static final long serialVersionUID = 8040572156190517223L;
	

	public static final int CONTROL_SEND_IMAGE = 10;
	public static final int CONTROL_PAN_LEFT = 1;
	public static final int CONTROL_PAN_RIGHT = 2;
	public static final int CONTROL_ZOOM_IN = 3;
	public static final int CONTROL_ZOOM_OUT = 4;
	
	/**
	 * Constructs a camera packet for control transfer
	 * @param type Control signal 
	 */
	public CameraPacket(int type){
		this.type = type;
		this.serializedImage = null;
	}
	/**
	 * Constructs a camera packet for image data transfer with a serialized image
	 * @param type Control signal
	 * @param serializedImage Serialized <code>BufferedImage</code> instance
	 */
	public CameraPacket(int type, byte[] serializedImage) {
		this.type = type;
		this.serializedImage = serializedImage;
	}
	/**
	 * Construnts a camera packet for image data transfer with a <code>BufferedImage</code>
	 * @param type Control sianel
	 * @param img <code>BufferedImage</code> instance to be serialized
	 */
	public CameraPacket(int type, BufferedImage img){
		this.type = type;
		this.serializedImage = toBytes(img);
	}
	
	/**
	 * Returns a control signal of this instances
	 * @return Control signal of this instances
	 */
	public int getType() {
		return type;
	}

	/**
	 * Returns an image of this camera packet
	 * @return <code>BufferedImage</code> instance of this camera packet
	 */
	public BufferedImage getBufferedImage(){
		return toBufferedImage(serializedImage);
	}

	/**
	 * Converts a camera packet instance to a serialized camera packet.
	 * Serialized camera packets are byte arrays 
	 * @return Serialized camera packet in the form of byte array 
	 */
	public byte[] toBytes(){
		return toBytes(this);
	}
	/**
	 * Converts a serialized camera packet to a camera packet instnace
	 * @param data Serialized camera packet
	 * @return <code>CameraPacket</code> instance
	 */
	public static CameraPacket toPacket(byte[] data){
		ByteArrayInputStream bais = new ByteArrayInputStream(data);
		CameraPacket packet = null;
		try{
			ObjectInputStream ois = new ObjectInputStream(bais);
			packet = (CameraPacket)ois.readObject();
		}catch(IOException ioe){
			ioe.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return packet;
	}
	
	private byte[] toBytes(Object o){  
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] ret = null;
		try{  
			ObjectOutputStream oos = new ObjectOutputStream(baos);  
			oos.writeObject(o);  
			oos.flush();
			baos.flush();
			ret = baos.toByteArray();
		}catch(IOException ioe){  
			ioe.printStackTrace();  
		}  
		return ret;
	}  
	private byte[] toBytes(BufferedImage img){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] ret = null;
		try{
			ImageIO.write(img, "jpeg", baos);
			baos.flush();
			ret = baos.toByteArray();
		}catch(IOException ioe){
			ioe.printStackTrace();
		}catch(IllegalArgumentException iae){
			/* empty */
		}
		return ret;
		
	}
	
	private BufferedImage toBufferedImage(byte[] data){
		ByteArrayInputStream bios = new ByteArrayInputStream(data);
		BufferedImage img = null;
		
		try {
			img = ImageIO.read(bios);
		} catch (IOException e) {
			e.printStackTrace();
		}
		

		return img;
		
	}

}
