package safehome.device.netcamera;

import java.awt.image.BufferedImage;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;

import safehome.device.DeviceCamera;

/**
 * This class represents a camera peer.
 * <p>
 * A camera peer has a camera device and is connected to SafeHome core. 
 * It sends images from a camera device to SafeHome core and receives
 * control signals from SafeHome core. 
 *  
 * @author yhkim
 *
 */
public class CameraPeer implements Runnable{	

	private Socket socket;
	private DataOutputStream outStream = null;
	private DataInputStream inStream = null;
	private DeviceCamera camera;
	private boolean isOn;

	/**
	 * Constructs a camera peer. A camera peer has a camera device instance and a socket connected to SAfeHome core.
	 * @param safehomeCoreIP the host name of SafeHome core
	 * @param safehomeCorePort the port number
	 * @throws UnknownHostException if the IP address of the host could not be determined. 
	 * @throws IOException if an I/O error occurs when creating the socket. 
	 */
	CameraPeer(String safehomeCoreIP, int safehomeCorePort) throws UnknownHostException, IOException{ 
		socket = new Socket(safehomeCoreIP, safehomeCorePort);
		outStream = new DataOutputStream(socket.getOutputStream());
		inStream = new DataInputStream(socket.getInputStream());
		camera = new DeviceCamera();
		camera.setID(1);
		isOn = false;

	}


	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run(){
		BufferedImage image;


		while(true){
			synchronized(camera){
				/*
				 * Get a camera image.
				 */
				image = camera.getView();
			}
			/* Create a packet with CONTROL_SEND_IMAGE control signal and image from a camera device */
			CameraPacket packet = new CameraPacket(CameraPacket.CONTROL_SEND_IMAGE, image);

			System.out.println("size of image packet, " + packet + " = " + packet.toBytes().length);

			try {
				/* First send the size of the camera packet to be sent */
				outStream.writeInt(packet.toBytes().length);
				/* Send the serialized camera packet through packet.toBytes() method*/
				outStream.write(packet.toBytes());
			} catch(SocketException se){
				if (isOn){
					se.printStackTrace();
				}
			}catch (IOException e1) {
				e1.printStackTrace();
			} 


			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

	}


	/**
	 * Receives control signals from SafeHome core and does the corresponding actions  
	 */
	public void readControl() {

		while(true){
			int size;
			byte[] data = null;
			CameraPacket packet = null;

			try {
				/* First read the size of an incoming packet */
				size = inStream.readInt();
				System.out.println("packet size = " + size);
				data = new byte[size];
				/* Read size bytes from socket */
				inStream.readFully(data, 0, size);
				packet = CameraPacket.toPacket(data);
			} catch (IOException e) {

				e.printStackTrace();
			}

			System.out.println("packet type = " + packet.getType());
			/* Do actions according to the control signal */ 
			synchronized(camera){
				switch(packet.getType()){
				case CameraPacket.CONTROL_PAN_LEFT:
					camera.panLeft();
					break;
				case CameraPacket.CONTROL_PAN_RIGHT:
					camera.panRight();
					break;
				case CameraPacket.CONTROL_ZOOM_IN:
					camera.zoomIn();
					break;
				case CameraPacket.CONTROL_ZOOM_OUT:
					camera.zoomOut();
					break;
				}

			}
		}
	}
}


